#!/usr/bin/perl

use strict;
use CPAN;

# Ugly global variables:

# Where to get the description from
my $cpan_codex='/var/lib/sorcery/codex/test/perl-cpan';
my $address='Neuron <vlmarek@volny.cz>';

# Name of the module
my $name=shift;
$name=~s/-/::/g;
my $version;
my $description;
my $id;

# All other modules we depend on; key = name, value = version
# TODO: we ignore minimal version ...
my %deps;

# Where to put spells
my $build_base=`pwd`;
chomp $build_base;
$build_base.='/CPAN/perl-cpan/';

system "mkdir -p '$build_base'";

# List of spells which provide perl itself
my %default_spells;
open F, 'default_spells' or die "Can't open default_spells: $!\n";
while (<F>) {
	chomp;
	$default_spells{$_}=1;
}
close F;

# Iterate over all modules, to find the one we are looking for
my @mod=CPAN::Shell->expand('Module', $name);
die "No module '$name' found\n" unless @mod;
if (@mod!=1) {
	# We found more than one module ...
	print join "\n", map { $_->id } @mod;
	die "\nMore than one module found.\n";
}

# Module name we are working with
my $mod=$mod[0];

# Get some values from cpan
$version=$mod->cpan_version;
$description=$mod->description;
$id=$mod->id;

my $cpan_file=$mod->cpan_file;
if ($cpan_file eq 'N/A' or $cpan_file =~ /^Contact Author/) {
	die "This module does not exist on CPAN\n";
}

# Find in which distribution the module is
my $pack = $CPAN::META->instance('CPAN::Distribution',$cpan_file);
# Directory where the module is unpacked
my $dir;
$mod->get unless ($dir = $pack->dir);
die "Can't load/find module :(\n" unless ($dir ||= $pack->dir);
chdir $dir or die "Can't chdir to '$dir': $!\n";

# yes '' because Makefile.PL may ask questions ... (like LWP)
# TODO: Connect somehow questions to our questions file ?
#       What about optional dependencies then ?
open F, 'yes \'\' | perl Makefile.PL PRINT_PREREQ=1 |';
my $f=join '', <F>;
close F;

# Extract dependencies from the output
$deps{$1}=$2 while ($f=~m/perl\((.*?)\)>=(\d+\.?\d*)/gs);

# Find the archive
my $md5=$$pack{'localfile'};
# And run md5 over unpacked tar
if ($cpan_file=~/\.gz$/) {
	$md5=`gunzip -c "$md5" | md5sum`;
} elsif ($cpan_file=~/\.bz2$/) {
	$md5=`bunzip2 -c "$md5" | md5sum`;
} else {
	die "Unknown suffix: '$cpan_file'\n";
}
($md5)=$md5=~/^([a-f0-9]{32})\s/ or die "Can't extract md5 from '$md5'\n";

# Some debug ...
print "\n";
print "ID: $id\n";
print "Version: $version\n";
print "Description: $description\n";
print "Dir: $dir\n";
print "File: $cpan_file\n";
print "MD5: $md5\n";

# Create directory for our new spell
my $spell_dir=$id;
$spell_dir=~s/::/-/g;

system "rm -rf '$build_base/$spell_dir'";
mkdir "$build_base/$spell_dir" or die "Can't create directory '$build_base/$spell_dir': $!\n";
chdir "$build_base/$spell_dir" or die "Can't enter directory '$build_base/$spell_dir': $!\n";

# Create files we need
DETAILS ();
BUILD ();
SORT_DETAILS ();
DEPENDS ();
TRIGGERS ();
HISTORY ();
chmod 0775, 'DETAILS', 'BUILD', 'DEPENDS', 'TRIGGERS';

sub HISTORY {
	my $hist;
	my $id=$id;
	$id=~s/::/-/g;
	if ( open F, "$cpan_codex/$id/HISTORY" ) {
		$hist=join '', <F>;
		close F;
	}
	my @t=localtime;
	my $t=sprintf "%04d-%02d-%02d", 1900+$t[5], 1+$t[4], $t[3];

	if (length $hist) {
		$hist="$t $address\n\t* Whatever\n\n$hist";
	} else {
		$hist="$t $address\n\t* All files: Initial version\n";
	}
	open F, '>HISTORY' or die "Can't open HISTORY: $!\n";
	print F $hist;
	close F;
}

sub TRIGGERS {
	open F, '>TRIGGERS' or die "Can't open DEPENDS: $!\n";
	print F "on_cast perl cast_self\n";
	close F;
}

sub DEPENDS {
	open F, '>DEPENDS' or die "Can't open DEPENDS: $!\n";

	# Module always depends on perl
	my @deps='perl';
	my $max=0;

	for (keys %deps) {
		# Throw out what provides perl itself
		next if defined $default_spells{$_};
		s/::/-/g;
		push @deps, $_;
		print "Dependency: $_\n";
		# Count lenght to do some pretty printing later
		$max=length $_ if length $_ > $max;
	}

	print F join "&&\n", map { "depends $_".' 'x(1+$max-length $_) } @deps;
	print F "\n";

	close F;
}

# Put all lines so that '=' match in one column
sub SORT_DETAILS {
	my $max=0;
	open F, 'DETAILS' or die "Can't open DETAILS: $!\n";
	(length $1 > $max) and $max=length $1 while (<F>=~m/^(.*?)=/);
	close F;
	open F, 'DETAILS' or die "Can't open DETAILS: $!\n";
	open G, '>DETAILS2' or die "Can't open DETAILS2: $!\n";
	print G ' 'x($max-length $1), "$1=$2\n" while ($_=<F>, m/^(.*?)=(.*)/);
	print G $_, <F>;
	close G;
	close F;
	rename 'DETAILS2', 'DETAILS' or die "Can't rename DETAILS2 to DETAILS: $!\n";
	system 'cat DETAILS';
}

sub DETAILS {
	open F, '>DETAILS' or die "Can't open DETAILS: $!\n";
	my $oldid=$id;
	my $id=$id;
	$id=~s/::/-/g;
	print F "SPELL=$id\n";
	print F "VERSION=$version\n";
	my ($source)=$cpan_file=~m#([^/]*)$#;
	$source=~s/$id/\${SPELL}/;
	$source=~s/$version/\${VERSION}/;
	print F "SOURCE=$source\n";
	my ($dir)=$dir=~m#([^/]*)$#;
	$dir=~s/$id/\${SPELL}/;
	$dir=~s/$version/\${VERSION}/;
	print F "SOURCE_DIRECTORY=\${BUILD_DIRECTORY}/$dir\n";
	my ($source2)=$cpan_file=~m#([^/]*)$#;
	$source2='http://www.cpan.org/authors/id/'.$cpan_file;
	$source2=~s/$id/\${SPELL}/;
	$source2=~s/$version/\${VERSION}/;
	my $pos;
	substr ($source2, $pos, length $source, '$SOURCE') if (($pos=index ($source2, $source))>=0);
	print F "SOURCE_URL[0]=$source2\n";
	print F "MD5[0]=$md5\n";
	print F "LICENSE=UNKNOWN\n";
	print F "WEB_SITE=http://search.cpan.org/search?dist=$id\n";
	my @t=localtime;
	printf F "ENTERED=%04d%02d%02d\n", 1900+$t[5], 1+$t[4], $t[3];
	printf F "UPDATED=%04d%02d%02d\n", 1900+$t[5], 1+$t[4], $t[3];

	my $short_cpan="SHORT=\"$description\"\n";
	my $short_smgl=`grep -E 'SHORT=' "$cpan_codex/$id/DETAILS"`;
	$short_smgl=~s/^\s*//;
	if (length $short_smgl and $short_smgl ne $short_cpan) {
		print "Two shorts:\nCPAN: ${short_cpan}SMGL: $short_smgl";
	}
	$short_cpan=$short_smgl if length $short_smgl;
	print F length $short_smgl ? $short_smgl : $short_cpan;

	my $desc="blabla";
	chomp ($desc=`$cpan_codex/$id/DETAILS`) if ( -x "$cpan_codex/$id/DETAILS" );

	print "DESC: $desc\n";
	print F <<EOT
cat << EOF
$desc
EOF
EOT
;
	close F;
}

sub BUILD {
	open F, '>BUILD' or die "Can't open BUILD: $!\n";
	print F <<EOT
perl Makefile.PL &&
make             &&
make test        &&
prepare_install  &&
make install
EOT
;
	close F;
}

# vim: cindent:autowrite:sw=3:ts=3
